(******************************************************************************
*
* ml : edge type definition and relative operations.
*
* Giulio Manzonetto, Giulio Marcon, Anna Sperotto
* <{gmanzone,gmarcon,asperott}@dsi.unive.it>
*
* 
*******************************************************************************)

module Edge = struct

    (* Data types and constructors *)

    type point = Point of int * int

    type theta = int

    type magnitude = int

    type gradient = Gradient of theta * magnitude 

    type edge = Edge of point * gradient

    type ndirections = int

    type edgelist = ndirections * edge list

    (* Operations *)

    (* sort_edges : sorts an edgelist by angle, x and y*)
    let sort_edges ( ndirs, edges ) = 
        List.sort
            (
                fun
                    ( Edge ( Point ( x1, y1 ), Gradient ( t1, _ ) ) )
                    ( Edge ( Point ( x2, y2 ), Gradient ( t2, _ ) ) )
                ->
                    if ( ( t1 mod ( ndirs / 2 ) ) = ( t2 mod ( ndirs / 2 ) ) ) then
                        if ( x1 = x2 ) then
                            if ( y1 = y2 ) then
                                0
                            else if ( y1 < y2 ) then
                                -1
                            else
                                1
                        else if ( x1 < x2 ) then
                            -1
                        else
                            1
                    else if ( t1 mod ( ndirs / 2 ) < t2 mod ( ndirs / 2 ) ) then
                        -1
                    else
                        1
            )
            edges

end

