(******************************************************************************
*
* logicalLinear.ml : detects edges with logical linear operators.
*
* Giulio Manzonetto, Giulio Marcon, Anna Sperotto
* <{gmanzone,gmarcon,asperott}@dsi.unive.it>
*
* 
*******************************************************************************)

open Image
open LogicalLinear
open LogicalLinear

module LogicalLinearEdgeOperator = struct

let edge sigma epsilon =
    let scale1 = 2.0 /. ( sqrt Math.pi ) /. 3.0 /. sigma in
    let scale2 = scale1 *. sigma *. sigma /. 4.0 /. epsilon in
    let scale3 = scale1 *. ( sigma ** 4.0 ) *. ( sqrt 2.0 ) /. 192.0 /. epsilon in
    LlandN ([
        (* n'_c *)
        Phi ( fun x -> scale1 *. ( Math.dG sigma x ) ) ;
        (* n''_l *)
        Phi ( fun x -> scale2 *. Math.d2G sigma ( x -. epsilon ) ) ;
        (* n''_r *)
        Phi ( fun x -> -. scale2 *. Math.d2G sigma ( x +. epsilon ) ) ;
        (* n^4_l *)
        Phi ( fun x -> -. scale3 *. Math.d4G ( sigma /. ( sqrt 2.0 ) ) ( x -. epsilon ) ) ;
        (* n^4_r *)
        Phi ( fun x -> scale3 *. Math.d4G ( sigma /. ( sqrt 2.0 ) )( x +. epsilon ) ) ;
    ])

let tangential2 op sigma ro stab =
    let ro = ro *. sigma in
    let smooth_partition x ro =
        let f x = exp ( -. 1.0 /. x) in
        let x = x *. ro in
        if x <= -0.5 then
            0.0
        else if x >= 0.5 then
            1.0
        else
            let f0 = f ( 0.5 +. x ) in
            f0 /. ( f0 +. ( f ( 0.5 -. x ) ) )
    in
    Phi (
        fun x ->
            ( Math.gaussian sigma x ) *.
            ( smooth_partition ( op x ) ro ) +.
            -. ( op ( stab ) ) *. 
            sigma *. ( Math.dG sigma x )
    )

let tangential sigma ro stab =
    LlandN ([
        tangential2 ( fun x -> -. x ) sigma ro stab ;
        tangential2 ( fun x -> x ) sigma ro stab
    ])

end
