(******************************************************************************
*
* main.ml: main program
*
* Giulio Manzonetto, Giulio Marcon, Anna Sperotto
* <{gmanzone,gmarcon,asperott}@dsi.unive.it>
*
* 
*******************************************************************************)

open Edgel
open Edge
open InteractionCoefficients
open MeanFieldAnnealing
open Ps
open SimulatedAnnealing
open SymmetricMatrix
open Utils

;;

let tmp = "tmp/" in
let ( k, alpha , edgenum, filename ) = Arghandle.annealing_get_parameters () in
let _ = Printf.printf "reading edges... %!" in
let (ndirs, edge_list) = Ps.read_file ( filename ^ ".ps" )in
let _ = Printf.printf "converting %d edges into edgels... \n%!" ( List.length edge_list ) in
let ( ndirs, edgels) =  Edgel.edgels_of_edges ( ndirs, edge_list ) in
let _ = Printf.printf "done, %d edgels created\nremoving edgels with less than %d edge... %!" ( List.length edgels ) edgenum in
let ( edgels, _ ) = List.partition (fun ( Edgel.Edgel ( _, _, edges ) ) -> ( List.length edges ) >= edgenum ) edgels in
let _ = Printf.printf "done, %d remaining edgels\ncreating interaction matrix... %!" ( List.length edgels ) in
let _ = 
    if ( !Arghandle.save ) then
	    let _ = Printf.printf "done, \nimage saving...\n%!" in	
        let ( _, edges ) = Edgel.edges_of_edgels ( ndirs, edgels ) in
	    Ps.write_file (filename ^ ".edge.ps") (ndirs, edges)
    else
	    ()
in
let ( _, edges ) = 
    if ( !Arghandle.sa ) then
        let c_matrix =  coefficients_matrix ~k  edgels   in
        (*
            let _ = Printf.printf "done\nmarshalling... %!" in
            let _ = Utils.marshal ( tmp ^ "marshalled_c.tmp" ) c_matrix in
            let _ = Printf.printf "done\nunmarshalling... %!" in
            let c_matrix = ( ( Utils.unmarshal ( tmp ^ "marshalled_c.tmp" ) : float array array ) ) in
        *)
        let _ = Printf.printf "done\nSimulated Annealing... \n%!" in
        let ( iterations, spin_vector ) = simulated_annealing alpha c_matrix in
        let _ = Printf.printf "done\ngetting figure and writing out... %!" in
        let edgels = get_figure_sim spin_vector edgels in
        let _ = Printf.printf "done, figure has %d edgels %d\n%!" ( List.length edgels ) in
        Edgel.edges_of_edgels ( ndirs, edgels )        
    else 
        let ( j, delta ) = get_energy_function_params ~k:k ~alpha:alpha edgels in
        (*
            let _ = Printf.printf "done\nmarshalling... %!" in
            let _ = Utils.marshal ( tmp ^ "marshalled_delta.tmp" ) delta in
            let _ = Utils.marshal ( tmp ^ "marshalled_j.tmp" ) j in
            let _ = Printf.printf "done\nunmarshalling... %!" in
            let delta = ( ( Utils.unmarshal ( tmp ^ "marshalled_delta.tmp" ) ) : float array ) in
            let j = ( ( Utils.unmarshal ( tmp ^ "marshalled_j.tmp" ) : float array array ) ) in
        *)
        let _ = Printf.printf "done\nMean Field Annealing... \n%!" in
        let ( iterations, spin_vector ) = 
            ( if ( !Arghandle.syn ) then mean_field_annealing_syn else mean_field_annealing_asyn ) 
            delta
            j 
        in
        let _ = Printf.printf "done\ngetting figure and writing out... %!" in
        let edgels = get_figure spin_vector edgels in 
        let _ = Printf.printf "done, figure has %d edgels %d\n%!" ( List.length edgels ) in
        Edgel.edges_of_edgels ( ndirs, edgels ) 
in
Ps.write_file (filename ^ ".out.ps") (ndirs, edges)
