(******************************************************************************
*
* mainParallel.ml: main program for parallel version of camleye
*
* Giulio Manzonetto, Giulio Marcon, Anna Sperotto
* <{gmanzone,gmarcon,asperott}@dsi.unive.it>
*
* 
*******************************************************************************)

open Edgel
open Edge
open InteractionCoefficients
open ParallelMeanFieldAnnealing
open ParallelUtils
open Ps
open SymmetricMatrix
open Utils

;;

(*
    k: dev'essere maggiore di 0
    alpha: piu' grande e' alpha, piu' alto e' il numero da cui parte sum_mu
*)
let ParallelUtils.MpiInfo ( comm, rank, size ) = ParallelUtils.parallel_init in 
let tmp = "" in
let ( k, alpha , edgenum, filename ) = Arghandle.parallel_get_parameters () in
let (ndirs, edgels, j , delta) =
    if ( rank = 0 ) then
        let _ = Printf.printf "reading edges... %!" in
        let (ndirs, edge_list) = Ps.read_file ( filename ^ ".ps" )in
        let _ = Printf.printf "converting %d edges into edgels... \n%!" ( List.length edge_list ) in
        let ( ndirs, edgels) =  Edgel.edgels_of_edges ( ndirs, edge_list ) in
        let _ = Printf.printf "done, %d edgels created\nremoving edgels with less than %d edge... %!" ( List.length edgels ) edgenum in
        let ( edgels, _ ) = List.partition (fun ( Edgel.Edgel ( _, _, edges ) ) -> ( List.length edges ) >= edgenum ) edgels in
        let _ = Printf.printf "done, %d remaining edgels\ncreating interaction matrix... \n%!" ( List.length edgels ) in
        let ( j, delta ) = get_energy_function_params ~k:k ~alpha:alpha edgels in
        ( ndirs, edgels, j, delta )
     else 
        ( 0, [], [||], [||] )
in
let ( j, delta ) = ParallelUtils.starting ( ParallelUtils.MpiInfo ( comm, rank, size ) ) ( j, delta ) in
let ( iterations, spin_vector ) = mean_field_annealing_syn ( ParallelUtils.MpiInfo ( comm, rank, size ) ) delta j in
if rank = 0 then
    let _ = Printf.printf "done\ngetting figure and writing out... %!" in
    let edgels = get_figure spin_vector edgels in
    let _ = Printf.printf "done, figure has %d edgels %d\n%!" ( List.length edgels ) in
    let ( _, edges ) = Edgel.edges_of_edgels ( ndirs, edgels ) in
    Ps.write_file (filename ^ ".out.ps") (ndirs, edges)
else 
    ()
