(******************************************************************************
*
* noise.ml: creates noise in an image 
*
* Giulio Manzonetto, Giulio Marcon, Anna Sperotto
* <{gmanzone,gmarcon,asperott}@dsi.unive.it>
*
* 
*******************************************************************************)

open Edge
open Ps

let sources = ref []
let file = ref ""
let percent = ref 0.0

let _ = Random.init ( int_of_float ( Unix.time () ) )

let usage = "Short Sighted Camel\n"
              ^ "Usage: " ^ Sys.argv.(0) ^ " file p\n"
              ^ "  file\t\t ps file containing edges\n"
              ^ "  p\t\t percent of noise (0..1)\n\n"

let _ =
    let anon s = sources := !sources @ [s] in
    let _ = Arg.parse [] anon usage in
    if ( ( List.length !sources ) == 2 ) then
        let _ = file := List.hd !sources in
        let _ = sources := List.tl !sources in
        let _ = percent := float_of_string ( List.hd !sources ) in
        let len = String.length !file in
        if ( len > 3 ) && ( (String.sub !file ( len - 3 ) 3 ) = ".ps" ) then
            file := String.sub !file 0 (len - 3) 
        else
            ( Printf.eprintf "%s" ( "wrong " ^ !file ^ " extension." ); exit 1 )
    else
        ( 
            Arg.usage [] usage; 
            exit 0 
        )

let add_noise ( ( ndirs, edges ) : Edge.edgelist ) max_x max_y num = 
    let rec _add_noise edges n =
        if n = 0 then
            edges
        else
            ( Edge.Edge ( Edge.Point ( ( Random.int max_x ), ( Random.int max_y ) ), Edge.Gradient ( ( Random.int ndirs ), ( Random.int 256 ) ) ) ) :: ( _add_noise edges ( n - 1 ) )
    in
    ( ndirs, _add_noise edges num )

let max_xy ( ndirs, edges ) = 
    let rec _max_xy edges max_x max_y = 
        match edges with
            | [] ->
                ( max_x, max_y ) 
            | ( Edge.Edge ( Edge.Point ( x, y ), _ ) ) :: tail ->
                _max_xy tail ( max x max_x ) ( max y max_y )
    in
    _max_xy edges 0 0 

;;

let _ = Printf.printf "reading edges... %!" in
let (ndirs, edges ) = Ps.read_file ( !file ^ ".ps" )in
let ( max_x, max_y ) = max_xy (ndirs, edges) in
let _ = Printf.printf "creating noise... %!" in
let ( _, edges ) = add_noise ( ndirs, edges ) max_x max_y ( int_of_float ( !percent *. ( float_of_int ( max_x * max_y ) ) ) ) in
let _ = Printf.printf "writing output... %!" in
Ps.write_file (!file ^ ".noise.ps") (ndirs, edges)
