(******************************************************************************
*
* parallelUtils.ml: Utilities for parallel execution
*
* Giulio Manzonetto, Giulio Marcon, Anna Sperotto
* <{gmanzone,gmarcon,asperott}@dsi.unive.it>
*
* 
*******************************************************************************)

open SymmetricMatrix

module ParallelUtils = struct
    
    type mpi_info =
        MpiInfo of ( Mpi.communicator * Mpi.rank * int )

    (* mpi info initialization *)
    let parallel_init =
        let comm = Mpi.comm_world in
        let size = ( Mpi.comm_size comm ) in
        let rank = ( Mpi.comm_rank comm ) in
        MpiInfo ( comm, rank, size )

    let generate_scatter ( MpiInfo ( comm, rank, size ) ) j delta =
        let n = Array.length delta in
        let m = n / size in
        let last_m = m + ( n mod m ) in
        Array.init size ( fun r ->
            ( Array.init ( if r = size - 1 then last_m else m ) 
                ( fun i -> 
                    Array.init ( SymmetricMatrix.length j ) ( 
                        fun k -> SymmetricMatrix.get j i k ) )
              , delta ) )

   let starting ( MpiInfo ( comm, rank, size ) ) ( j, delta ) =
        let marray =
            if rank = 0 then
                generate_scatter ( MpiInfo ( comm, rank, size ) ) j delta
            else
                [||]
        in
        Mpi.scatter marray 0 comm 

end
