(******************************************************************************
*
* ps.ml : ps reader/writer
*
* Giulio Manzonetto, Giulio Marcon, Anna Sperotto
* <{gmanzone,gmarcon,asperott}@dsi.unive.it>
*
* 
*******************************************************************************)

open Edge

exception Invalid_Format of string

module Ps = struct

    let read_file file : Edge.edgelist =
    let ib = Scanf.Scanning.from_file file in
    let rec read_file_r () =
        try
        let element =
            Scanf.bscanf
            ib
            "%d %d %d %d edge\n"
            (
                fun x y d i ->
                Edge.Edge
                (
                    Edge.Point ( x, y ),
                    Edge.Gradient ( d , i )
                )
            )
        in
        element :: read_file_r ()
        with
        | End_of_file ->
            []
        | Scanf.Scan_failure e ->
            Scanf.bscanf
                ib
                "%s@\n"
                (
                fun s -> read_file_r ()
                )
    in
    let rec read_ndirs_r () =
        try
            Scanf.bscanf
            ib
            "%d initcoords\n"
            (
            fun ndirs -> ( ( ndirs * 2 ), read_file_r ())
            )
        with
        | End_of_file ->
            raise ( Invalid_Format "Unexpected End_of_file" )
        | Scanf.Scan_failure e ->
            Scanf.bscanf
                ib
                "%s@\n"
                (
                fun s -> read_ndirs_r ()
                )
    in
    read_ndirs_r ()

    let write_file file ( ndirs, edges ) =
    let out = open_out file in
    let max_x_y edges =
        let rec _max_x_y relmax_x relmax_y edges =
            match edges with
                | [] ->
                    ( relmax_x, relmax_y ) 
                | Edge.Edge ( Edge.Point ( x, y ), Edge.Gradient ( d, i ) ) :: tail ->
                    _max_x_y ( max relmax_x x ) ( max relmax_y y ) tail
        in
        _max_x_y 0 0 edges
    in
    let (max_x, max_y) = max_x_y edges in
    let _ = Printf.fprintf 
        out 
        "%%!PS-Adobe-2.0 EPSF-2.0\n%%%%BoundingBox: 0 0 %d %d\n%%%%Pages: 1\n%%%%EndProlog\n%%%%Page: 1 1\ngsave [1 0 0 -1 0 %d] concat\n512 dict begin\n%% <ndirs> initcoords \n/initcoords { \n %% ndirs \n 2 mul \n %% 2*ndirs \n dup array /x0 exch def \n dup array /y0 exch def \n dup array /x1 exch def \n dup array /y1 exch def \n %% 2*ndirs \n dup 360 exch div \n%% 2*ndirs 180/ndirs \n 0 1 3 index 1 sub { \n  %% 2*ndirs 180/ndirs i \n  dup 2 index mul \n  %% 2*ndirs 180/ndirs i i*180/ndirs \n  dup cos 0.75 mul \n  dup 0.5 add x0 4 index 3 -1 roll put \n  neg 0.5 add x1 3 index 3 -1 roll put \n  %% 2*ndirs 180/ndirs i i*180/ndirs \n  sin 0.75 mul \n  dup 0.5 add y0 3 index 3 -1 roll put \n  neg 0.5 add y1 2 index 3 -1 roll put \n  %% 2*ndirs 180/ndirs i \n  pop \n } for \n %% 2*ndirs 180/ndirs \n pop pop \n} bind def \n \n%% <x> <y> <dir> coords \n/coords { \n %% x y dir \n 3 copy \n %% x y dir x y dir \n dup y0 exch get exch x0 exch get  \n %% x y dir x y y0 x0 \n 4 -1 roll add  \n %% x y dir y y0 x+x0 \n 6 1 roll add 4 1 roll \n %% x+x0 y+y0 x y dir \n dup y1 exch get exch x1 exch get \n %% x+x0 y+y0 x y y1 x1 \n 4 -1 roll add \n %% x+x0 y+y0 y y1 x+x1 \n 3 1 roll add \n %% x+x0 y+y0 x+x1 y+y1 \n} bind def \n \n%% <x> <y> <dir> <conf> edge \n/edge { \n 0.3 setlinewidth \n edgeconf coords \n %% x+x0 y+y0 x+x1 y+y1 \n moveto lineto stroke \n} bind def \n \n%% <x> <y> <dir> <conf> plin \n/plin { \n 0.2 setlinewidth \n plinconf coords \n %% x+x0 y+y0 x+x1 y+y1 \n moveto lineto stroke \n} bind def \n \n%% <x> <y> <dir> <conf> nlin \n/nlin { \n 0.2 setlinewidth \n nlinconf coords \n %% x+x0 y+y0 x+x1 y+y1 \n moveto lineto stroke \n} bind def \n \n%% <conf> setconf \n/b0conf { -510 div 0.5   add setgray} bind def \n/b1conf {-1020 div 0.25  add setgray} bind def \n/w1conf { 1020 div 0.75  add setgray} bind def \n/g1conf {-1020 div 0.375 add setgray} bind def \n \n\n%d initcoords\n/edgeconf /b0conf load def\n%%labels:\n" 
        ( max_x + 1) 
        ( max_y + 1) 
        ( max_y + 1) 
        ( ndirs / 2 ) 
    in
    let rec write_file_r = function
        | [] ->
            let _ = Printf.fprintf out "end grestore showpage\n%%%%Trailer\n" in
            close_out out 
        | Edge.Edge ( Edge.Point ( x, y ), Edge.Gradient ( d, i ) ) :: edges ->
            let _ = Printf.fprintf out "%d %d %d %d edge\n" x y d i in
            write_file_r edges
    in
    write_file_r edges

end
